@@ -12,15 +12,17 @@ from math import ceil
 from operator import eq, ne
 from pathlib import Path
 from shutil import move
-from typing import List
+from typing import List, Optional, Tuple, Union
+from sys import exit
 
 from tabulate import tabulate
 
 from .attachment import Attachment
 from .contacts import Contacts
 from .config import Config, get_terminal_size
+from .definition import Settings
 from .dialogue import Cancelled, is_yes, ask
-from .field import Field
+from .field import Cell, Field
 from .identifier import Identifier
 from .informer import Informer
 from .mail_draft import MailDraft
@@ -32,49 +34,43 @@ logger = logging.getLogger(__name__)
 
 
 class Parser:
-    is_split: bool  
+    is_split: bool
     "has already been split into many files (or we are just about to start file processing)"
-    is_analyzed: bool  
+    is_analyzed: bool
     "has already been analyzed"
-    is_processable: bool  
+    is_processable: bool
     "there are known actions we are ready to perform"
-    is_processed: bool 
+    is_processed: bool
     "has already been processed"
     attachments: List[Attachment]
 
     def __init__(self, source_file: Path = False, stdin=None, types=None, prepare=True):
         self.is_formatted = False
         self.is_repeating = False
-        self.dialect = None  
+        self.dialect = None
         "CSV dialect"
         self.has_header = None
         "CSV has header"
         self.is_pandoc = False
         "pandoc table format (second line to be skipped, a lot of spaces)"
-        self.header: str = "" 
+        self.header: str = ""
         "if CSV has header, it's here so that Processor can take it"
-        self.sample: List[str] = []  
-        "lines  of the original file, including first line - possible header"
-        self.sample_parsed: List[List[str]] = []
-        "values of the prepared output (ex re-sorted), always excluding header"
-        self.fields: List[Field] = []  
-        "CSV columns that will be generated to an output"
-        self.first_line_fields: List[str] = []  
+        self.sample: List[str] = []
+        "lines of the original file, including first line - possible header"
+        self.sample_parsed: List[List[Cell]] = []
+        """Values of the prepared output (ex re-sorted), always excluding header.
+        Note that Field.compute_preview might append here a list instead of a str, hence the type Cell. """
+        self.fields: List[Field] = []
+        "CSV columns that will be generated to an output. Only parser.add_field is used to append to this list."
+        self.first_line_fields: List[str] = []
         "CSV columns (equal to header if used) in the original file"
-        self.types = []  
+        self.types = []
         "field types of the columns as given by the user"
-        # settings:
-        #    * "add": new_field:Field,
-        #             source_col_i:int - number of field to compute from,
-        #             fitting_type:Field - possible type of source ,
-        #             custom:tuple - If target is a 'custom' field, we'll receive a tuple (module path, method name).
-        #    * "dialect": always present (set in controller just after parser.prepare()), output CSV dialect
-        #    * "header": True if input CSV has header and output CSV should have it too.
-        #                False if either input CSV has not header or the output CSV should omit it.
-        #
-        self.settings = defaultdict(list)
+        self.settings: Settings = defaultdict(list)
+        "processing settings"
         self.redo_invalids = Config.get("redo_invalids")
-        self.otrs_cookie = False  # OTRS attributes to be linked to CSV
+        # OTRS attributes to be linked to CSV
+        self.otrs_cookie = False
         self.otrs_id = Config.get("ticketid", "OTRS")
         self.otrs_token = False
         self.otrs_num = Config.get("ticketnum", "OTRS")
@@ -83,7 +79,8 @@ class Parser:
             self.attachment_name += ".csv"
         self.ip_count_guess = None
         self.ip_count = None
-        self.attachments = []  # files created if splitting
+        self.attachments = []
+        "files created if splitting"
         self.queued_lines_count: int
         self.invalid_lines_count: int
         self.unknown_lines_count: int
@@ -93,30 +90,32 @@ class Parser:
         # XX this is not used right now, convey is not at the moment connectable to other programs
         # see __init__.py at revision d2cf88f48409ca8cc5e229954df34836de884445
         self.external_stdout = None
-        self.is_single_query = False  # CSV processing vs single_query check usage
+        self.is_single_query = False
+        "CSV processing vs single_query check usage"
         self.ranges = {}  # XX should be refactored as part of Whois
         self.ip_seen = {}  # XX should be refactored as part of Whois
-        # set by processor [location file][grouped row][order in aggregation settings] = [sum generator, count]
         self.aggregation = defaultdict(dict)
+        "set by processor [location file][grouped row][order in aggregation settings] = [sum generator, count]"
         self.refresh()
         self._reset(reset_header=False)
-        self.selected: List[int] = [] 
+        self.selected: List[int] = []
         "list of selected fields col_i that may be in/excluded and moved in the menu"
-        self.files_created = set()  
+        self.files_created = set()
         "files created with this parser, will not be rewritten but appended to if reprocessing lines"
 
         # load CSV
         self.source_file: Path = source_file or self.invent_file_str()
         self.stdin = []
-        # When accepting input from stdin and not saving the output into a file
-        #   or when setting this to True,
-        #   the output will be here.
-        # Config.get("output") - the file we should save to
-        # Config.get("stdout") - stdout used
         self.stdout = Config.get("stdout")
+        """When accepting input from stdin and not saving the output into a file
+          or when setting this to True,
+          the output will be here.
+        Config.get("output") - the file we should save to
+        Config.get("stdout") - stdout used"""
         self.stdout_sample = None
         self.target_file = None
-        self.saved_to_disk = None  # has been saved to self.target_file
+        self.saved_to_disk = None
+        "has been saved to self.target_file"
         self.processor = Processor(self)
         self.informer = Informer(self)
         self.identifier = Identifier(self)
@@ -143,7 +142,7 @@ class Parser:
     def prepare(self):
         if self.size == 0:
             print("Empty contents.")
-            quit()
+            exit()
         self.prepare_target_file()
 
         # check if we are parsing a single cell
@@ -177,12 +176,11 @@ class Parser:
             elif not len_ or len_ > 1:
                 seems = False
 
-            if seems and Config.get("single_query") is not False:
+            if Config.get("single_query") or (seems and Config.get("single_query") is not False):
                 # identify_fields some basic parameters
                 self.add_field([Field(self.stdin[0])])  # stdin has single field
                 self.dialect = False
                 self.has_header = False
-                self.sample_parsed = [x for x in csv.reader(self.sample)]
                 if self.identifier.identify_fields(quiet=True):
                     # tell the user what type we think their input is
                     # access the detection message for the first (and supposedly only) field
@@ -207,6 +205,7 @@ class Parser:
                     return self
 
         # we are parsing a CSV file
+        self.sample_parsed = [x for x in csv.reader(self.sample)]
         self.informer.sout_info()
         try:
             # Dialog to obtain basic information about CSV - delimiter, header
@@ -264,7 +263,7 @@ class Parser:
             self.reset_settings()
         except Cancelled:
             print("Cancelled.")
-            quit()
+            exit()
         self.informer.sout_info()
         self.is_formatted = True  # delimiter and header has been detected etc.
         return self
@@ -292,7 +291,7 @@ class Parser:
             fields.append((field, s))
         return fields
 
-    def add_field(self, replace: List["Field"] = None, append: "Field" = None):
+    def add_field(self, replace: Optional[List[Field]] = None, append: Optional[Field] = None):
         """
         :param replace: Replace fields by the new field list.
         :param append: Append new field to the current list.
@@ -308,7 +307,7 @@ class Parser:
             f.col_i_original = f.col_i = len(self.fields)
             if self.types and len(self.types) > f.col_i:
                 f.type = self.types[f.col_i]
-            f.parser = self
+            f.set_parser(self)  # register Field to this parser
             self.fields.append(f)
         return self
 
@@ -318,7 +317,7 @@ class Parser:
                 self.types = [getattr(Types, t) for t in types.split(",")]
             except AttributeError:
                 print(f"Unknown type amongst: {types}")
-                quit()
+                exit()
         else:
             self.types = []
         return self
@@ -390,17 +389,15 @@ class Parser:
                         fields.append((target_type, methods))
 
         method_cache = {}
+        single_query_value = "\n".join(self.stdin)
+        row = [single_query_value]
         for field, methods in fields:
             if type(field) is Type:
-                val = self.first_line
-            elif not self.sample_parsed[0]:
-                print(f"No field to compute {field} from.")
-                return
+                val = single_query_value
             else:
-                val = self.sample_parsed[0][field.source_field.col_i]
+                val = row[field.source_field.col_i]
             try:
                 for i, l in enumerate(methods):
-                    val_old = val
                     if (repr((val, methods[:i + 1]))) in method_cache:
                         val = method_cache[(repr((val, methods[:i + 1])))]
                         continue
@@ -414,7 +411,7 @@ class Parser:
                     method_cache[(repr((val, methods[:i + 1])))] = val
             except Exception as e:
                 val = str(e)
-            self.sample_parsed[0].append(val)
+            row.append(val)
             if type(field) is Type or field.is_chosen:
                 append(field, val)
 
@@ -534,10 +531,10 @@ class Parser:
             l.append(str(f))
         agg = se["aggregate"]
         if agg:
-            if agg[0] is not None:
-                l.append(f"{self.fields[agg[0]]}-grouped")
-            for fn, col in agg[1]:
-                l.append(f"{fn.__name__}-{self.fields[col]}")
+            if agg.group_by is not None:
+                l.append(f"{agg.group_by}-grouped")
+            for fn, col in agg.actions:
+                l.append(f"{fn.__name__}-{col}")
         if hasattr(self, "source_file"):
             l.insert(0, self.source_file.stem)
             target_file = f"{'_'.join(l)}{self.source_file.suffix}"
@@ -824,7 +821,7 @@ class Parser:
         rows = []  # nice table formatting
         full_rows = []  # formatting that optically matches the Sample above
         for line in self.sample_parsed:
-            row = []
+            row: List[Tuple[str,Field]] = []
             for field, cell in zip_longest(self.fields, line):
                 if field is None:
                     # when lines have different length, field may be none
@@ -834,7 +831,7 @@ class Parser:
                     cell = field.compute_preview(line)
                 # suppress new lines in preview; however while processing,
                 # new lines are printed and this may render the CSV unloadable
-                row.append((cell.replace("\n", r"\n"), field))
+                row.append((str(cell).replace("\n", r"\n"), field))
 
             # check if current line is filtered out
             line_chosen = True
@@ -858,6 +855,18 @@ class Parser:
             full_rows.append([*g(False)])
         return full_rows, rows
 
+    def get_similar(self, fields2: Union[Field, List[Field]]):
+        """ Recommend which fields might be similar in different parsers.
+            Based on the same column type.
+
+            (In the future, we might implement comparing the similarity by the parser sample values too.)
+        """
+        if isinstance(fields2, Field):
+            fields2 = [fields2]
+        return [f1 for f1 in self.fields if f1.type for f2 in fields2 if f1.type == f2.type]
+
+
+
     def __getstate__(self):
         state = self.__dict__.copy()
         del state['informer']
