@@ -1,4 +1,4 @@
-from __future__ import annotations  # remove as of Python3.11
+from __future__ import annotations  # remove as of Python3.11 in every file
 import io
 import logging
 import traceback
@@ -9,9 +9,11 @@ from functools import reduce
 from operator import eq, ne, mul
 from pathlib import Path
 from queue import Queue, Empty
+from sys import exit
 from threading import Thread, Lock
-from typing import Dict, TYPE_CHECKING
+from typing import Dict, TYPE_CHECKING, List, Optional, Tuple, Union
 
+from .action import Expandable
 from .attachment import Attachment
 from .config import Config
 from .dialogue import ask
@@ -19,7 +21,10 @@ from .types import Web
 from .whois import Quota, UnknownValue
 
 if TYPE_CHECKING:
+    import _csv
     from .parser import Parser
+    from .definition import Settings
+
 
 logger = logging.getLogger(__name__)
 
@@ -28,10 +33,12 @@ def prod(iterable):  # XX as of Python3.8, replace with math.prod
     return reduce(mul, iterable, 1)
 
 
+
+
 class Processor:
     """ Opens the CSV file and processes the lines. """
 
-    descriptors: Dict[str, object]  # location => file_descriptor, his csv-writer
+    descriptors: Dict[Union[int, str], Tuple[io.StringIO, _csv._writer]]  # location => file_descriptor, his csv-writer
     descriptorsStatsOpen: Dict[str, int]  # {location: count} XX(performance) we may use a SortedDict object
 
     def __init__(self, parser, rewrite=True):
@@ -59,11 +66,15 @@ class Processor:
 
         # apply setup
         # convert settings["add"] to lambdas
-        adds = []
-        for f in settings["add"]:  # [("netname", 20, [lambda x, lambda x...]), ...]
-            adds.append((f.name, f.source_field.col_i_original, f.get_methods()))
+        # Ex: ... = [("netname", 20, [lambda x, lambda x...]), ...]
+        settings["addByMethod"] = [(f.name, f.source_field.col_i_original, f.get_methods()) for f in settings["add"]]
         del settings["add"]
-        settings["addByMethod"] = adds
+
+        # convert settings["merge"] to lambdas
+        if settings["merge"]:
+            settings["merging"] = True
+            settings["addByMethod"] += [("merged", op.local_column.col_i_original, (lambda x: op.get(x),)) for op in settings["merge"]]
+            del settings["merge"]
 
         # convert filter settings to pre (before line processing) and post that spare a lot of time
         #   (ex: skip lines before WHOIS processing)
@@ -161,7 +172,8 @@ class Processor:
                     if o == "a":
                         Config.set("lacnic_quota_skip_lines", True, "FIELDS")
                     if o == "d":
-                        print("Maybe you should hit n multiple times because pdb takes you to the wrong scope.")  # I dont know why.
+                        # I dont know why.
+                        print("Maybe you should hit n multiple times because pdb takes you to the wrong scope.")
                         Config.get_debugger().set_trace()
                     elif o == "s":
                         continue  # skip to the next line XXmaybe this is not true for threads
@@ -177,7 +189,7 @@ class Processor:
                         return
                     elif o == "q":
                         self._close_descriptors()
-                        quit()
+                        exit()
                     else:  # continue from last row
                         parser.line_count -= 1  # let's pretend we didn't just do this row before and give it a second chance
                         if thread_count:
@@ -262,7 +274,7 @@ class Processor:
         for f in self.descriptors.values():
             f[0].close()
 
-    def process_line(self, parser: Parser, line, settings, fields=None):
+    def process_line(self, parser: Parser, line: List, settings: Settings, fields: Union[Tuple, List]=None):
         """
         Parses line – compute fields while adding, perform filters, pick or delete cols, split and write to a file.
         """
@@ -292,9 +304,9 @@ class Processor:
             # add fields
             if add:
                 list_lengths = []
-                for col_i in settings["addByMethod"]:  # [("netname", 20, [lambda x, lambda x...]), ...]
-                    val = fields[col_i[1]]
-                    for l in col_i[2]:
+                for _, col_i, lambdas in settings["addByMethod"]:  # [("netname", 20, [lambda x, lambda x...]), ...]
+                    val = fields[col_i]
+                    for l in lambdas:
                         if isinstance(val, list):
                             # resolve all items, while flattening any list encountered
                             val = [y for x in (l(v) for v in val) for y in (x if type(x) is list else [x])]
@@ -311,13 +323,17 @@ class Processor:
                         # if we received empty list, row is invalid
                         # ex: missing IP of a hostname
                         if not fields[i]:
-                            raise RuntimeWarning(f"Column {i + 1} cannot be determined and the row was marked as invalid")
+                            raise RuntimeWarning(
+                                f"Column {i + 1} cannot be determined and the row was marked as invalid")
                         fields[i] *= row_count // len(fields[i])
                     it = zip(*fields)
                     fields = it.__next__()  # now we are sure fields has only scalar values
                     for v in it:  # duplicate row because one of lambdas produced a multiple value list
                         self.process_line(parser, line, settings, v)  # new row with scalar values only
 
+            if settings["merging"]:
+                fields = tuple(Expandable.flatten(fields))
+
             # post filtering
             # filter (include, col, value), ex: [(True, 23, "passed-value"), (False, 13, "another-value")]
             for include, col_i, val in settings["f_post"]:
@@ -341,7 +357,8 @@ class Processor:
 
             # determine location
             if type(settings["split"]) == int:
-                location = fields[settings["split"]].replace("/", "-")  # split ('/' is a forbidden char in linux file names)
+                # split ('/' is a forbidden char in linux file names)
+                location = fields[settings["split"]].replace("/", "-")
                 if not location:
                     parser.unknown_lines_count += 1
                     location = Config.UNKNOWN_NAME
@@ -351,15 +368,8 @@ class Processor:
 
             # aggregation: count column
             if settings["aggregate"]:
-                # settings["aggregate"] = column to be grouped, [(sum, column to be summed)]
-                # Ex: settings["aggregate"] = 1, [(Aggregate.sum, 2), (Aggregate.avg, 3)]
-                # Ex: settings["aggregate"] = 0, [(Aggregate.count, 0)]
-                # Ex: settings["aggregate"] = None, [(Aggregate.sum, 1)]
-                # Ex: settings["aggregate"] = None, [(Aggregate.sum, 1), (Aggregate.avg, 1)]
-
-                col_group_i = settings["aggregate"][0]
-                grp = fields[col_group_i] if col_group_i is not None else None
-                for i, (fn, col_data_i) in enumerate(settings["aggregate"][1]):
+                grp = fields[settings["aggregate"].group_by.col_i] if settings["aggregate"].group_by else None
+                for i, (fn, col_data) in enumerate(settings["aggregate"].actions):
                     # counters[location file][grouped row][order in aggregation settings] = [sum generator, count]
                     loc = self.parser.aggregation[location]
 
@@ -372,14 +382,14 @@ class Processor:
                         if fn.__name__ == "list":
                             loc[None][i][1] = "(all)"  # we do not want to enlist whole table
                         else:
-                            loc[None][i][1] = loc[None][i][0].send(fields[col_data_i])
+                            loc[None][i][1] = loc[None][i][0].send(fields[col_data.col_i])
 
                     if grp not in loc:
                         loc[grp] = []
                     if len(loc[grp]) <= i:
                         loc[grp].append([fn(), 0])
                         next(loc[grp][i][0])
-                    loc[grp][i][1] = loc[grp][i][0].send(fields[col_data_i])
+                    loc[grp][i][1] = loc[grp][i][0].send(fields[col_data.col_i])
                 return  # we will not write anything right know, aggregation results are not ready yet
         except Quota.QuotaExceeded:
             parser.queued_lines_count += 1
