@@ -10,16 +10,18 @@ from contextlib import contextmanager
 from csv import writer
 from os import linesep
 from pathlib import Path
+from sys import exit
 from time import time
 
 import ezodf
 import jsonpickle
+import openpyxl
 import xlrd
 from netaddr import IPRange
 from xlrd import XLRDError
 
 from .config import Config, config_dir
-from .dialogue import is_yes
+from .dialogue import hit_any_key, is_yes
 from .identifier import Identifier
 from .parser import Parser
 from .utils import lazy_print
@@ -83,7 +85,7 @@ class Wrapper:
             stdin = file_or_input.split("\n") if file_or_input else read_stdin()
         elif force_file:
             file = file_or_input if file_or_input else choose_file()
-        elif file_or_input and len(file_or_input) < 256 and Path(file_or_input).is_file():
+        elif isinstance(file_or_input, Path) or (file_or_input and len(file_or_input) < 256 and Path(file_or_input).is_file()):
             # if longer than 255, it is most probably not a file but input
             file = file_or_input
         elif file_or_input:
@@ -106,7 +108,7 @@ class Wrapper:
 
         if not file and not stdin:
             print("No input. Program exit.")
-            quit()
+            exit()
         elif stdin:
             Config.set_cache_dir(Path.cwd())
             self.cache_file = None
@@ -116,7 +118,7 @@ class Wrapper:
 
         if not Path(file).is_file():
             print(f"File '{file}' not found.")
-            quit()
+            exit()
 
         self.assure_cache_file(file)
         if self.cache_file.is_file() and not (fresh or reprocess):
@@ -138,9 +140,9 @@ class Wrapper:
                 if self.parser.is_analyzed or self.parser.is_formatted:
                     self.parser.informer.sout_info()
                     logger.info("It seems the file has already been formatted.")
-            except BdbQuit:  # we do not want to catch quit() signal from pdb
+            except BdbQuit:  # we do not want to catch the exit signal from pdb
                 print("Stopping.")
-                quit()
+                exit()
             except Exception as e:
                 print(e)
                 print("Format of the file may have changed since last time. "
@@ -255,11 +257,10 @@ class Wrapper:
             # so that I could raise an official issue.
             jsonpickle.decode(string, keys=True)
         except Exception:
-            print("The program state is not picklable by 'jsonpickle' module. "
+            hit_any_key("The program state is not picklable by 'jsonpickle' module. "
                   "Continuing will provide a file that will have to be reanalysed. "
                   "You may post this as a bug to the project issue tracker.")
             Config.error_caught()
-            input("Continue...")
 
         # save cache file
         if self.cache_file:  # cache_file does not exist typically if reading from STDIN
@@ -297,7 +298,7 @@ class Wrapper:
             self.cache_file.write_text(string)  # save cache
 
     def clear(self):
-        self.check_xls() or self.check_ods() or self.check_log()
+        self.check_ods() or self.check_xlsx() or self.check_xls() or self.check_log()
 
         self.parser = Parser(self.file, self.stdin, self.types)
         self.save()
@@ -356,8 +357,8 @@ class Wrapper:
         """ Check if the contents is a XLS file """
         try:
             wb = xlrd.open_workbook(self.file)
-            sh = wb.sheet_by_name('Sheet1')
-        except XLRDError:
+            sh = wb.sheets()[0]
+        except (XLRDError, IndexError):
             pass
         else:
             if is_yes("This seems like a XLS file. Do you wish to transform it to CSV first?"):
@@ -366,6 +367,21 @@ class Wrapper:
                         target.writerow(sh.row_values(row))
             return True
 
+    def check_xlsx(self):
+        """ Check if the contents is a XLSX file """
+        try:
+            wb = openpyxl.load_workbook(self.file, read_only=True, keep_vba=False, data_only=True, keep_links=False)
+            sh = wb[wb.sheetnames[0]]
+        except (openpyxl.utils.exceptions.InvalidFileException, IndexError):
+            pass
+        else:
+            if is_yes("This seems like a XLSX file. Do you wish to transform it to CSV first?"):
+                with self.rework() as target:
+                    for row in sh.values:
+                        target.writerow(row)
+            return True
+
+
     @contextmanager
     def rework(self):
         target = Path(str(self.file) + ".csv")
@@ -382,15 +398,17 @@ class Wrapper:
                     return
             else:
                 print(f"The file {target} already exist, cannot be recreated.")
-                quit()
+                exit()
         # noinspection PyBroadException
         try:
             with target.open("w") as f:
                 yield writer(f)
         except Exception as e:
-            print(e)
-            input(f"Could not convert. Hit any key.")
+            print(f"Could not convert.")
+            logger.error(e)
+            Config.error_caught()
+            raise
         else:
-            input(f"Successfully written to {target.absolute()}. Hit any key.")
+            hit_any_key(f"Successfully written to {target.absolute()}.")
             # XX remove old cache directory if empty
             self.assure_cache_file(target.absolute())  # changes self.file
